"""adding TTT Stats to the database

Revision ID: 666a5a89e2b0
Revises: d34f5f07944f
Create Date: 2025-06-24 00:27:47.896011

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '666a5a89e2b0'
down_revision: Union[str, None] = 'd34f5f07944f'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('ttt_stats',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('user_id', sa.BigInteger(), nullable=True),
    sa.Column('wins', sa.Integer(), nullable=False),
    sa.Column('losses', sa.Integer(), nullable=False),
    sa.Column('games_played', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['user_id'], ['users.user_id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('user_id')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('ttt_stats')
    # ### end Alembic commands ###


"""adding card card schema, updated user, added badges

Revision ID: 17393111d12e
Revises: e976e9d6d502
Create Date: 2026-01-05 01:38:00.794015

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '17393111d12e'
down_revision: Union[str, Sequence[str], None] = 'e976e9d6d502'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



# This file is part of discord-shared-db
#
# Copyright (C) 2026 CouchComfy
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from sqlalchemy import Enum as SQEnum, Identity
from sqlalchemy.orm import relationship
from sqlalchemy import BigInteger, Column, ForeignKey, DateTime, String, UniqueConstraint

from discord_shared_db.base import Base
from discord_shared_db.auth_enums import AuthProviders


class UserAuth(Base):
    __tablename__ = "user_auth"


    id = Column(BigInteger, Identity(start=1), primary_key=True)
    user_id = Column(String, ForeignKey("users.user_id"), nullable=False)

    provider = Column(SQEnum(AuthProviders), nullable=False)
    provider_account_id = Column(String, nullable=False)

    linked_date = Column(DateTime, nullable=False)

    user = relationship("User", back_populates="auth_methods")

    __table_args__ = (
        UniqueConstraint("provider", "provider_account_id"),
    )


'''
user_auth
---------
user_id
provider      -- "steam" | "discord"
provider_id   -- SteamID64 or Discord user ID
linked_at

'''


"""Update User and add Pixel Data

Revision ID: 8e015abbe587
Revises: 666a5a89e2b0
Create Date: 2025-07-27 23:41:45.188635

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '8e015abbe587'
down_revision: Union[str, Sequence[str], None] = '666a5a89e2b0'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



# This file is part of discord-shared-db
#
# Copyright (C) 2026 CouchComfy
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from sqlalchemy.orm import relationship
from sqlalchemy import BigInteger, Boolean, Column, ForeignKey, Identity, Integer, String

from discord_shared_db.base import Base

class Deck(Base):
    __tablename__ = "decks"

    id = Column(Integer, Identity(start=1), primary_key=True)
    user_id = Column(String, ForeignKey("users.user_id"))
    name = Column(String, nullable=False)
    is_active = Column(Boolean, default=False)

    user = relationship("User", back_populates="decks")
    cards = relationship("DeckCard", cascade="all, delete-orphan")


class DeckCard(Base):
    __tablename__ = "deck_cards"

    deck_id = Column(Integer, ForeignKey("decks.id"), primary_key=True)
    user_card_id = Column(Integer, ForeignKey("user_cards.id"), primary_key=True)

    deck = relationship("Deck", back_populates="cards")
    user_card = relationship("UserCard")




# This file is part of discord-shared-db
#
# Copyright (C) 2026 CouchComfy
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import logging
from datetime import datetime
from functools import wraps

def log_time(message="Function completed in"):
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            logging.info(f"⏳ Starting: {func.__name__}")
            start_time = datetime.now()
            result = await func(*args, **kwargs)
            end_time = datetime.now()
            total_time = (end_time - start_time).total_seconds()
            logging.info(f"{message} {total_time:.3f} seconds")
            return result
        return wrapper
    return decorator


# This file is part of discord-shared-db
#
# Copyright (C) 2026 CouchComfy
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from sqlalchemy.orm import relationship
from sqlalchemy import JSON, BigInteger, Boolean, Column, DateTime, ForeignKey, Identity, Integer, String

from discord_shared_db.base import Base

class UserCard(Base):
    __tablename__ = "user_cards"

    id = Column(Integer, Identity(start=1), primary_key=True)
    user_id = Column(String, ForeignKey("users.user_id"), nullable=False)

    raw_card_id = Column(String, ForeignKey("raw_cards.id"), nullable=False)

    pull_number = Column(Integer, nullable=False)
    modifiers = Column(JSON, default=list)  # foils, alt-art, buffs, etc.
    obtained_at = Column(DateTime, nullable=False)

    used_in_deck = Column(Boolean, nullable=False)

    user = relationship("User", back_populates="cards")




# This file is part of discord-shared-db
#
# Copyright (C) 2026 CouchComfy
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

from enum import Enum

from sqlalchemy.orm import relationship
from sqlalchemy import Enum as SQEnum, Identity
from sqlalchemy import JSON, Column, ForeignKey, Integer, String

from discord_shared_db.base import Base
from discord_shared_db.card_game.tcg_enum import ActionCardType, ManaType, Rarity

class RawCard(Base):
    __tablename__ = "raw_cards"

    id = Column(String, primary_key=True)
    name = Column(String, nullable=False)
    collect_number = Column(Integer, nullable=False)
    rarity = Column(SQEnum(Rarity), nullable=False)
    image_path = Column(String, nullable=False)
    flavor_text = Column(String)
    
    amount_pulled = Column(Integer, default=0)

    card_type = Column(String, nullable=False)

    __mapper_args__ = {
        "polymorphic_on": card_type,
        "polymorphic_identity": "base",
    }

class RawMove(Base):
    __tablename__ = "raw_moves"

    id = Column(Integer, Identity(start=1), primary_key=True,)
    card_id = Column(String, ForeignKey("raw_cards.id"), nullable=False)

    name = Column(String, nullable=False)
    mana_cost = Column(JSON, nullable=False)
    effect_text = Column(String, nullable=False)
    effect_code = Column(String, nullable=False)

    card = relationship("RawCharacterCard", back_populates="moves")

 
class RawCharacterCard(RawCard):
    __tablename__ = "raw_character_cards"

    id = Column(String, ForeignKey("raw_cards.id"), primary_key=True)

    total_energy = Column(Integer, nullable=False)
    mana_type = Column(SQEnum(ManaType), nullable=False)

    moves = relationship(
        "RawMove",
        back_populates="card",
        cascade="all, delete-orphan",
    )

    __mapper_args__ = {
        "polymorphic_identity": "character",
    }

class RawActionCard(RawCard):
    __tablename__ = "raw_action_cards"

    id = Column(String, ForeignKey("raw_cards.id"), primary_key=True)

    action_card_type = Column(SQEnum(ActionCardType), nullable=False)
    mana_cost = Column(JSON)
    effect_text = Column(String)
    effect_code = Column(String)

    __mapper_args__ = {
        "polymorphic_identity": "action",
    }


class RawLocationCard(RawCard):
    __tablename__ = "raw_location_cards"

    id = Column(String, ForeignKey("raw_cards.id"), primary_key=True)

    mana_cost = Column(JSON)
    effect_text = Column(String)
    effect_code = Column(String)

    __mapper_args__ = {
        "polymorphic_identity": "location",
    }
    





"""add pixels_data table

Revision ID: 281181d752d4
Revises: b58357e1fc2e
Create Date: 2025-07-29 12:17:10.814367

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '281181d752d4'
down_revision: Union[str, Sequence[str], None] = 'b58357e1fc2e'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('pixels_data',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('user_id', sa.BigInteger(), nullable=True),
    sa.Column('color', sa.Enum('PINK', 'RED', 'ORANGE', 'YELLOW', 'GREEN', 'BLUE', 'PURPLE', 'BROWN', 'BLACK', 'WHITE', 'TAN1', 'TAN2', name='colors'), nullable=False),
    sa.Column('dd_time', sa.DateTime(), nullable=False),
    sa.Column('x', sa.Integer(), nullable=False),
    sa.Column('y', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['user_id'], ['users.user_id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('pixels_data')
    # ### end Alembic commands ###
